@@ -80,10 +80,14 @@ Example::
 '''
 import logging
 import sys
+import httplib
+import time
+import pyxb
+import d1_common.types.exceptions
 
 # D1
 try:
-  import d1_common.restclient as restclient
+  from d1_client import d1baseclient
 except ImportError as e:
   sys.stderr.write('Import error: {0}\n'.format(str(e)))
   sys.stderr.write('Try: easy_install DataONE_Common\n')
@@ -115,8 +119,11 @@ class ObjectListIterator(object):
     :param max: Maximum number of items to retrieve (all)
     :type max: integer
     '''
+    self.log = logging.getLogger(self.__class__.__name__)
     self._objectList = None
     self._czero = 0
+    self._citem = 0
+    self._pageoffs = 0
     self._client = client
     if max >= 0 and max < pagesize:
       pagesize = max
@@ -141,27 +148,47 @@ class ObjectListIterator(object):
 
   def next(self):
     '''Implements the next() method for the iterator.  Returns the next
-    ObjectInfo instance.
+    ObjectInfo instance. Loads more if at the end of the page and there's more
+    pages to load.
     '''
+    self.log.debug("%d / %d (%d)" % (self._citem, self._maxitem, len(self._objectList.objectInfo)))
     if self._citem >= self._maxitem:
       raise StopIteration
-    if self._citem >= len(self._objectList.objectInfo):
+    if (self._pageoffs) >= len(self._objectList.objectInfo):
       self._loadMore(start = self._czero+len( self._objectList.objectInfo ))
       if len( self._objectList.objectInfo ) < 1:
         raise StopIteration
-    res = self._objectList.objectInfo[self._citem]
+    res = self._objectList.objectInfo[ self._pageoffs ]
     self._citem += 1
+    self._pageoffs += 1
     return res
 
 
-  def _loadMore(self, start=0):
+  def _loadMore(self, start=0, trys=0, validation=True):
     '''Retrieves the next page of results
     '''
+    self.log.debug("Loading page starting from %d" % start)
     self._czero = start
-    self._citem = 0
-    self._objectList = self._client.listObjects(start = start,
-                                                count = self._pagesize,
-                                                fromDate = self.fromDate)
+    self._pageoffs = 0
+    try:
+      pyxb.RequireValidWhenParsing(validation)
+      self._objectList = self._client.listObjects(start = start,
+                                                  count = self._pagesize,
+                                                  fromDate = self.fromDate)
+    except httplib.BadStatusLine as e:
+      self.log.warn("Server responded with Bad Status Line. Retrying in 5sec")
+      self._client.connection.close()
+      if trys > 3:
+        raise e
+      trys += 1
+      self._loadMore(start, trys)
+    except d1_common.types.exceptions.ServiceFailure as e:
+      self.log.error(e)
+      if trys > 3:
+        raise e
+      trys += 1
+      self._loadMore(start, trys, validation=False)
+    self._client.connection.close()
 
 
   def __len__(self):
@@ -171,12 +198,13 @@ class ObjectListIterator(object):
 
 
 #===============================================================================
+
 if __name__ == "__main__":
   '''A simple demonstration of the iterator.  Walks over the list of objects
   available from a given node. Output is in YAML.
   '''
   parser = OptionParser()
-  default_base_url = 'https://mn-unm-1.dataone.org/mn'
+  default_base_url = 'https://cn.dataone.org/cn'
   parser.add_option('-b','--base_url', dest='base_url', default=default_base_url,
                  help='ListObjects from BASEURL (default=%s)' % default_base_url)
   parser.add_option('-l', '--loglevel', dest='llevel', default=20, type='int',
@@ -184,30 +212,30 @@ if __name__ == "__main__":
                      '40=Error, 50=Fatal')
   parser.add_option('-s', '--start', dest='start', default=0, type='int',
                  help='Start retrieving objects from this position (default=0')
-  parser.add_option('-m','--max', dest='max', default=-1, type='int',
-                 help='Maximum number of entries to retrieve (all)')
-  parser.add_option('-p','--page', dest='pagesize', default=500, type='int',
-                 help='Maximum number of entries to retrieve per call (500)')
+  parser.add_option('-m','--max', dest='max', default=500, type='int',
+                 help='Maximum number of entries to retrieve (500)')
+  parser.add_option('-p','--page', dest='pagesize', default=100, type='int',
+                 help='Maximum number of entries to retrieve per call (100)')
   (options, args) = parser.parse_args()
   if options.llevel not in [10,20,30,40,50]:
     options.llevel = 20
   logging.basicConfig(level=int(options.llevel))
 
-  client = restclient.DataONEBaseClient(options.base_url)
+  client = d1baseclient.DataONEBaseClient(options.base_url)
   ol = ObjectListIterator(client, start=options.start,
                           pagesize=options.pagesize,
                           max=options.max)
-  counter = 0
+  counter = options.start
   print "---"
   print "total: %d" % len(ol)
   print "---"
   for o in ol:
-    counter += 1
     print "-"
     print "  item     : %d" % counter
     print "  pid      : %s" % o.identifier.value()
     print "  modified : %s" % o.dateSysMetadataModified
-    print "  format   : %s" % o.objectFormat
+    print "  format   : %s" % o.formatId
     print "  size     : %s" % o.size
     print "  checksum : %s" % o.checksum.value()
     print "  algorithm: %s" % o.checksum.algorithm
+    counter += 1
