@@ -944,7 +944,8 @@ class SOLRSearchResponseIterator(object):
   retrieve all the values.
   '''
   def __init__(self, client, q, fq=None, fields='*', pagesize=100,
-               transformer=SOLRRecordTransformer()):
+               transformer=SOLRRecordTransformer(),
+               max_records=1000):
     '''
     Initialize.
 
@@ -959,6 +960,9 @@ class SOLRSearchResponseIterator(object):
     self.q = q
     self.fq = fq
     self.fields = fields
+    if max_records is None:
+      max_records = 99999999
+    self.max_records = max_records
     self.crecord = 0
     self.pagesize = pagesize
     self.res = None
@@ -974,10 +978,13 @@ class SOLRSearchResponseIterator(object):
     Retrieves the next set of results from the service.
     '''
     self.logger.debug("Iterator crecord=%s" % str(self.crecord))
+    pagesize = self.pagesize
+    if (offset + pagesize) > self.max_records:
+      pagesize = self.max_records - offset 
     params = {
       'q': self.q,
       'start':str(offset),
-      'rows':str(self.pagesize),
+      'rows':str(pagesize),
       'fl':self.fields,
       'explainOther':'',
       'hl.fl':''
@@ -1002,6 +1009,9 @@ class SOLRSearchResponseIterator(object):
   def next(self):
     if self.done:
       raise StopIteration()
+    if self.crecord > self.max_records:
+      self.done = True
+      raise StopIteration()
     idx = self.crecord - self.res['response']['start']
     try:
       row = self.res['response']['docs'][idx]
